//
// Tiny OS with critical section and delays.
//

#include "lpc2378.h"
#include "init.h"
#include "print.h"
#include "os.h"

// Allocate stacks one for each task
int stackOne[STACKSIZE];
int stackTwo[STACKSIZE];
int stackIdle[STACKSIZE];

void taskOne(void);   // Task one counts up from 0
void taskTwo(void);   // Task two counts down from 0xFFFFFFFF
void idleTask(void);  // Idle task


///////////////////////////////////////////////////////////////////////////////////////////
//
//  main
//
//  Creates three tasks with CreateTask which initialize their TaskBlocks and stacks.
//  Starts the timer for the interrupt ticks which cause the tasks to switch.
//  Calls StartOS with sets the current task and switches to it.
//
//  This function never runs passed the call to StartOS() so it never exits.
//
int main(void)
{
    InitHardware();

    printString("University of Washington - Context_Switcher Test Application \n");

    // Create tasks
    CreateTask(0, stackOne, (void*)taskOne);
    CreateTask(1, stackTwo, (void*)taskTwo);
    CreateTask(2, stackIdle, (void*)idleTask);

    InitializeTimerTick();

    StartOS();

    // Never gets here.
    printString("Main completed.\n");
    while(1);
    return 0;
}


