#include "lpc2378.h"
#include "print.h"
#include "os.h"

//
// hello_world with timer reset
//
void hello_world(void)
{
	WRITEREG32(T0IR, 0xFF);         // Reset timer
	printString("Hello world!\n");
}

//
// taskOne counts up from 0.
// Never exits.
//
void taskOne(void)
{
	CPU_SR cpu_sr = 0;

	int count = 0;
	while(1)
	{
		//>> Protect with critical section
		printString("task one: ");
		print_uint32(count++);
		printString("\n");

		//>> Replace with DelayTask
		for(int i = 0; i < 10000; i++);    // delay
	}
}

//
// taskTwo counts down from 0xFFFFFFFF
// Never exits.
//
void taskTwo(void)
{
	CPU_SR cpu_sr = 0;

	int count = 0xFFFFFFFF;
	while(1)
	{
		//>> Protect with critical section
		printString("task two: ");
		print_uint32(count--);
		printString("\n");

		//>> Replace with DelayTask
		for(int i = 0; i < 10000; i++);    // delay
	}
}

//
// idleTask runs whenever the OS cannot any other tasks.
// For example if all tasks are delayed.
//
void idleTask(void)
{
	while(1)
	{
		printString("Idle Task");

		// Do not replace. The idleTask should not be delayed
		// Or there will be nothing to run.
		for(int i = 0; i < 10000; i++);    // delay
	}
}

